@@ -1,40 +1,32 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2001-2017 German Aerospace Center (DLR) and others.
-/****************************************************************************/
-//
-//   This program and the accompanying materials
-//   are made available under the terms of the Eclipse Public License v2.0
-//   which accompanies this distribution, and is available at
-//   http://www.eclipse.org/legal/epl-v20.html
-//
+// Copyright (C) 2001-2022 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 /****************************************************************************/
 /// @file    GUIPropertyScheme.h
 /// @author  Michael Behrisch
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Mon, 20.07.2009
-/// @version $Id$
 ///
 //
 /****************************************************************************/
-#ifndef GUIPropertyScheme_h
-#define GUIPropertyScheme_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
+#pragma once
 #include <config.h>
-#endif
 
 #include <cassert>
 #include <vector>
 #include <utils/common/RGBColor.h>
 #include <utils/iodevices/OutputDevice.h>
+#include <utils/gui/images/GUIIcons.h>
 
 
 // ===========================================================================
@@ -52,10 +44,14 @@ class GUIPropertyScheme {
 public:
     /// Constructor
     GUIPropertyScheme(const std::string& name, const T& baseColor,
-                      const std::string& colName = "", const bool isFixed = false, double baseValue = 0) :
+                      const std::string& colName = "", const bool isFixed = false, double baseValue = 0,
+                      RGBColor bgColor = RGBColor::WHITE,
+                      GUIIcon icon = GUIIcon::EMPTY) :
         myName(name), myIsInterpolated(!isFixed),
         myIsFixed(isFixed),
-        myAllowNegativeValues(false) {
+        myAllowNegativeValues(false),
+        myIcon(icon),
+        myBgColor(bgColor) {
         addColor(baseColor, baseValue, colName);
     }
 
@@ -168,11 +164,19 @@ public:
         myAllowNegativeValues = value;
     }
 
-    void save(OutputDevice& dev) const {
+    GUIIcon getIcon() const {
+        return myIcon;
+    }
+
+    const RGBColor& getBackgroundColor() const {
+        return myBgColor;
+    }
+
+    void save(OutputDevice& dev, const std::string& prefix = "") const {
         const std::string tag = getTagName(myColors);
 
         dev.openTag(tag);
-        dev.writeAttr(SUMO_ATTR_NAME, myName);
+        dev.writeAttr(SUMO_ATTR_NAME, prefix + myName);
         if (!myIsFixed) {
             dev.writeAttr(SUMO_ATTR_INTERPOLATED, myIsInterpolated);
         }
@@ -182,7 +186,7 @@ public:
         while (threshIt != myThresholds.end()) {
             dev.openTag(SUMO_TAG_ENTRY);
             dev.writeAttr(SUMO_ATTR_COLOR, *colIt);
-            if (!myIsFixed) {
+            if (!myIsFixed && (*threshIt) != std::numeric_limits<double>::max()) {
                 dev.writeAttr(SUMO_ATTR_THRESHOLD, *threshIt);
             }
             if ((*nameIt) != "") {
@@ -229,12 +233,10 @@ private:
     std::vector<std::string> myNames;
     bool myIsFixed;
     bool myAllowNegativeValues;
+    GUIIcon myIcon;
+    RGBColor myBgColor;
 
 };
 
 typedef GUIPropertyScheme<RGBColor> GUIColorScheme;
 typedef GUIPropertyScheme<double> GUIScaleScheme;
-
-#endif
-
-/****************************************************************************/
